@@ -3,11 +3,11 @@
 use crate::{file::load_file, get_context};
 use std::collections::HashMap;
 
-#[cfg(not(feature = "audio"))]
+#[cfg(any(not(feature = "audio"), target_os = "android"))]
 #[path = "audio/no_sound.rs"]
 mod snd;
 
-#[cfg(not(target_arch = "wasm32"))]
+#[cfg(not(any(target_arch = "wasm32", target_os = "android")))]
 #[cfg(feature = "audio")]
 #[path = "audio/native_snd.rs"]
 mod snd;
@@ -41,7 +41,13 @@ pub struct Sound(usize);
 /// Attempts to automatically detect the format of the source of data.
 pub async fn load_sound(path: &str) -> Result<Sound, crate::file::FileError> {
     let data = load_file(path).await?;
+    load_sound_from_bytes(&data).await
+}
 
+/// Load audio data.
+///
+/// Attempts to automatically detect the format of the source of data.
+pub async fn load_sound_from_bytes(data: &[u8]) -> Result<Sound, crate::file::FileError> {
     let sound = load_native_snd(&data).await;
 
     let ctx = &mut get_context().audio_context;
